#import "iEPGforFriio.h"
#include <uuid/uuid.h>
/*
BS/CS Channels(black type friio only):
 13 : BS-Asahi      24 : 110CS #1
 14 : BS-i          25 : 110CS #2
 15 : WOWOW         26 : 110CS #3
 16 : BS-Japan      27 : 110CS #4
 17 : BS11          28 : 110CS #5
 18 : Star Channel  29 : 110CS #6
 19 : TwellV        30 : 110CS #7
 20 : BS-NTV        31 : 110CS #8
 21 : BS-Fuji       32 : 110CS #9
 22 : NHK BS1/BS2   33 : 110CS #a
 23 : NHK hi        34 : 110CS #b
 35 : 110CS #c
b25: 
 b25 : use b25     nob25 : no use b25 
friiotype: 
 0 : Any   1 : White(digital)  2 : Black(BS/CS)
 
recfriio-wb recsec channel outputfile [b25] [friiotype [LocationID]]
recfriio-wb 15 13 a.ps b25 2 
 */ 

@implementation iEPGforFriio

/** initializer for our plug-in */
-(id) initWithArguments:(NSDictionary *)arguments {
	PDEBUG();
	if ((self = [super init]) != nil) {
		pluginArguments = [arguments copy];
	}
	return self;
}


/* initializer for our plug-in */
- (void) dealloc {
	PDEBUG();
    [pluginArguments release];
	[super dealloc];
}


/* WebPlugInViewFactory message handler for vending plug-in instances */
+ (NSView *)plugInViewWithArguments:(NSDictionary *)arguments {
    iEPGforFriio *thePlugin;
	PDEBUG();
	thePlugin = [[[self alloc] initWithArguments:arguments] autorelease];
    return thePlugin;
}


/* Load your nib file inside of your webPlugInInitialize method. */
- (void)webPlugInInitialize {
	BOOL result;
	PDEBUG();
	result = [NSBundle loadNibNamed:@"PluginView" owner:self];
}


/* adjust the frame boundaries inside of your awakeFromNib method */
- (void)awakeFromNib {
	PDEBUG();
	
	if( viewPanel != nil ) {
		[self setFrame:[viewPanel frame]]; /* adjust the frame from your nib's view to the plug-in's view */
		[self addSubview:viewPanel]; /* add the view from the nib as our one and only sub-view */
		
		NSDictionary *webPluginAttributesObj = [pluginArguments objectForKey:WebPlugInAttributesKey];
		NSString *URLString = [webPluginAttributesObj objectForKey:@"src"];
		if (URLString != nil && [URLString length] != 0) {
			NSURL *baseURL = [pluginArguments objectForKey:WebPlugInBaseURLKey];
			NSURL *URL = [NSURL URLWithString:URLString relativeToURL:baseURL];
			NSError *error;
			iEPG = [[NSString stringWithContentsOfURL:URL encoding:NSShiftJISStringEncoding error:&error] retain];
			[scriptText setSelectedRange:NSMakeRange(0, 3200)];
			if(!iEPG){
				[scriptText insertText:@"Failed to load iEPG data."];
				[scriptText insertText:[error description]];
			}
			else{
				[scriptText insertText:iEPG];
			}
		}
	}
}

- (void) backPage
{
	/* retrieve the plug-in's container */
	id pluginContainer = [pluginArguments objectForKey:WebPlugInContainerKey];
	if (pluginContainer) {	
		/* retrieve a reference to the webview */
		WebView *myWebView = [[pluginContainer webFrame] webView];
		[myWebView stringByEvaluatingJavaScriptFromString:@"history.back()"];
	}
}

-(IBAction) cancel:(id)sender {
	PDEBUG();
	[self backPage];
}

-(IBAction) runScript:(id)sender {
	PDEBUG();
	
	NSMutableDictionary *dic = [NSMutableDictionary dictionaryWithCapacity:11];
	for(NSString *l in [iEPG componentsSeparatedByString:@"\n"]){
		NSArray *a = [l componentsSeparatedByString:@": "];
		if([a count] == 2)
			[dic setObject:[[a objectAtIndex:1] stringByTrimmingCharactersInSet:[NSCharacterSet controlCharacterSet]] 
					forKey:[a objectAtIndex:0]];
	}
	int year = atoi([[dic objectForKey:@"year"] cStringUsingEncoding:NSUTF8StringEncoding]);
	int month = atoi([[dic objectForKey:@"month"] cStringUsingEncoding:NSUTF8StringEncoding]);
	int day = atoi([[dic objectForKey:@"date"] cStringUsingEncoding:NSUTF8StringEncoding]);
	int hourStart, hourEnd;
	int minStart, minEnd;
	sscanf([[dic objectForKey:@"start"] cStringUsingEncoding:NSUTF8StringEncoding], "%d:%d", &hourStart, &minStart);
	sscanf([[dic objectForKey:@"end"]   cStringUsingEncoding:NSUTF8StringEncoding], "%d:%d", &hourEnd,   &minEnd);
	
	NSString *station = nil;
	if([[dic objectForKey:@"version"] isEqualToString:@"2"])
		station = [dic objectForKey:@"station-name"];
	else
		station = [dic objectForKey:@"station"];
	
	NSString *dateStart = nil, *dateEnd = nil;
	
	time_t tStart, tEnd;
	struct tm t;
	time(&tStart);
	localtime_r(&tStart, &t);// get tm_zone, tm_gmtoff from current time
	
	t.tm_year = year - 1900;
	t.tm_mon = month - 1;
	t.tm_mday = day;
	t.tm_hour = hourStart;
	t.tm_min = minStart;
	t.tm_sec = 0;
	t.tm_isdst = 0;	
	tStart = mktime(&t);
	localtime_r(&tStart, &t);
	dateStart = [NSString stringWithFormat:@"%d/%d/%d %d:%d", 1900 + t.tm_year, 1 + t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min];
	
	tEnd = tStart;
	if(hourStart * 60 + minStart > hourEnd * 60 + minEnd) tEnd += 24 * 60 * 60;// next day ?
	localtime_r(&tEnd, &t);// get end date, time is not correct
	t.tm_hour = hourEnd;
	t.tm_min = minEnd;
	tEnd = mktime(&t);// re-calicurate end time
	dateEnd = [NSString stringWithFormat:@"%d/%d/%d %d:%d", 1900 + t.tm_year, 1 + t.tm_mon, t.tm_mday, t.tm_hour, t.tm_min];
	
	time_t durSec = tEnd - tStart; // recording seconds
	if([interval state]) durSec += 60; // v1.5
	if([shorten  state]) durSec -=  3; // v1.5
	
	// build launch script
	if(![[NSFileManager defaultManager] fileExistsAtPath:@"/usr/local/bin/recfriio"]){
		[scriptText setSelectedRange:NSMakeRange(0, 3200)];
		[scriptText insertText:@"/usr/local/bin/recfriio not found."];
		return;
	}
	
	NSString *scriptDir = [[NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES) objectAtIndex:0] 
						   stringByAppendingPathComponent:@"Friio/Scripts/"];
	if(![[NSFileManager defaultManager] fileExistsAtPath:scriptDir])
		[[NSFileManager defaultManager] createDirectoryAtPath:scriptDir withIntermediateDirectories:YES attributes:nil error:nil];
	NSString *scriptPath;
	
	NSString *recurrence;//, *deleteScript = @"";
	switch([[repeat selectedItem] tag]){		
		default: 
		recurrence = @" ";
		scriptPath = [scriptDir stringByAppendingPathComponent:[NSString stringWithFormat:@"/%04d%02d%02d-%02d%02d", year, month, day, hourStart, minStart]];
		//deleteScript = @"tell application \"Finder\" to delete \"%@\" as POSIX file\n";
		break;
		
		case 1: // every day
		recurrence = @", recurrence:\"FREQ=DAILY;INTERVAL=1\""; 
		scriptPath = [scriptDir stringByAppendingPathComponent:[NSString stringWithFormat:@"/%04d%02d%02d-%02d%02d-DAILY", year, month, day, hourStart, minStart]];
		break;
		
		case 2: // every week
		recurrence = @", recurrence:\"FREQ=WEEKLY;INTERVAL=1\"";
		scriptPath = [scriptDir stringByAppendingPathComponent:[NSString stringWithFormat:@"/%04d%02d%02d-%02d%02d-WEEKLY", year, month, day, hourStart, minStart]];
		break;
		
		case 3: // every month
		recurrence = @", recurrence:\"FREQ=MONTHLY;INTERVAL=1\"";
		scriptPath = [scriptDir stringByAppendingPathComponent:[NSString stringWithFormat:@"/%04d%02d%02d-%02d%02d-MONTHLY", year, month, day, hourStart, minStart]];
		break;
			//case 4: "RRULE:FREQ=YEARLY;UNTIL=19510505T170000Z;BYMONTH=5;BYDAY=1SU"
			//RRULE:FREQ=WEEKLY;INTERVAL=1;BYDAY=MO,TU,WE,TH,FR;WKST=MO		
	}
	
	// get channel from station
	int channel = 0, band = 0;	
	NSString *chPath = [[[NSBundle bundleWithIdentifier:@"friio.iEPGiCalFriio"] resourcePath] stringByAppendingPathComponent:@"channel.txt"];
	NSString *ch = [NSString stringWithContentsOfFile:chPath encoding:NSUTF8StringEncoding error:nil];
	for(NSString *l in [ch componentsSeparatedByString:@"\n"]){
		NSArray *a = [l componentsSeparatedByString:@":"];
		if([a count] == 3){
			if([station isEqualToString:[a objectAtIndex:1]]){
				channel = atoi([[a objectAtIndex:0] cStringUsingEncoding:NSUTF8StringEncoding]);
				band = atoi([[a objectAtIndex:2] cStringUsingEncoding:NSUTF8StringEncoding]);
				break;
			}
		}
	}
	if(channel == 0){
		[scriptText setSelectedRange:NSMakeRange(0, 3200)];
		[scriptText insertText:[NSString stringWithFormat:@"Failed to select channnel '%@' in channel.txt\n\n%@", station, ch]];
		return;
	}
	
/*
set ct to current date
set ds to (year of ct as string) & "-" & ((month of ct as number) as string) & "-" & (day of ct as string)
set cmd to "/usr/local/bin/recfriio -s %d -c %d -f '%@/Movies/" & ds & "%@' &> /dev/null &"
do shell script cmd
*/
	// make script file which will be launched by iCal and this will launch recfriio, then this will delete self
	NSString *moviePath = [NSString stringWithFormat:@"-%02d%02d-%02dCH-%@-%@.ts", 
								hourStart, minStart, channel, station, [dic objectForKey:@"program-title"]];
#if 0
[NSHomeDirectory() 
//stringByAppendingPathComponent:[NSString stringWithFormat:@"Movies/%04d%02d%02d-%02d%02d-%02dCH-%@-%@.ts", year, month, day, 
stringByAppendingPathComponent:[NSString stringWithFormat:@"-%02d%02d-%02dCH-%@-%@.ts", 
hourStart, minStart, channel, station, [dic objectForKey:@"program-title"]]];
#endif
	// set ct to current date\n set ds to "Movies/" & (year of ct as string) & "-" & ((month of ct as number) as string) & "-" & (day of ct as string)\n
	NSString *movieScript;
	if(band == 1) // 地上波
		movieScript = [NSString stringWithFormat:
					   @"set ct to current date\nset ds to (year of ct as string) & \"-\" & ((month of ct as number) as string) & \"-\" & (day of ct as string)\nset cmd to \"/usr/local/bin/recfriio -s %d -c %d -f '%@/Movies/\" & ds & \"%@' &> /dev/null &\"\ndo shell script cmd\n", durSec,  channel, NSHomeDirectory(), moviePath];

	else // BS
		movieScript = [NSString stringWithFormat:
					   @"set ct to current date\nset ds to (year of ct as string) & \"-\" & ((month of ct as number) as string) & \"-\" & (day of ct as string)\nset cmd to \"/usr/local/bin/recfriio-wb %d %d '%@/Movies/\" & ds & \"%@' b25 2 &> /dev/null &\"\ndo shell script cmd\n",  durSec,  channel, NSHomeDirectory(), moviePath];

	if(![movieScript writeToFile:[NSString stringWithFormat:@"%@.applescript", scriptPath] atomically:NO encoding:NSUTF8StringEncoding error:nil])
		NSLog(@"Failed to write script:%@", [NSString stringWithFormat:@"%@.applescript", scriptPath]);
	// convert applescript to scrt. is it possible in program ?
	int ret = system([[NSString stringWithFormat:@"/usr/bin/osacompile -d -o '%@.scrt' '%@.applescript'", scriptPath, scriptPath] 
			cStringUsingEncoding:NSUTF8StringEncoding]);
	if(ret)
		NSLog(@"Failed to execute osacompile:%d %@", ret, [[NSString stringWithFormat:@"/usr/bin/osacompile -d -o '%@.scrt' '%@.applescript'", scriptPath, scriptPath] 
			cStringUsingEncoding:NSUTF8StringEncoding]);
	[[NSFileManager defaultManager] removeItemAtPath:[NSString stringWithFormat:@"%@.applescript", scriptPath] error:nil];
	
	/*
	 tell application "iCal"
	 try
	 get calendar "Friio"
	 on error
	 make new calendar with properties {name:"Friio"}
	 end try	
	 set newEvent to make new event at end of events of calendar "Friio" with properties {summary:"TITLE", start date:date "7/15/2008 10:00", end date:date "7/15/2008 11:00"}
	 set theAlarm to make new open file alarm at end of open file alarms of newEvent with properties {trigger interval:-1, filepath:POSIX path of "/Users/takahiro/Downloads/startRecfriio/startRecfriio.app"}
	 end tell
	 */
	
	NSString *script = @"tell application \"iCal\"\nactivate\ntry\nget calendar \"Friio\"\non error\nmake new calendar with properties {name:\"Friio\"}\nend try\nset newEvent to make new event at end of events of calendar \"Friio\" with properties {summary:\"%@\", start date:date \"%@\", end date:date \"%@\"%@}\nset theAlarm to make new open file alarm at end of open file alarms of newEvent with properties {trigger interval:%d, filepath:POSIX path of \"%@\"}\nend tell\n";
	//NSLog(@"[interval state] = %d", [interval state]);
	
	// register to iCal
	NSAppleScript *scrpt = [[NSAppleScript alloc] init];	
	[scrpt initWithSource:[NSString stringWithFormat:script,
						   [NSString stringWithFormat:@"%@-%@", station, [dic objectForKey:@"program-title"]],
						   dateStart, dateEnd, recurrence,
						   [interval state] == 0 ? 0 : -1,
						   [NSString stringWithFormat:@"%@.scrt", scriptPath]]];
	//NSLog([scrpt source]);
	NSDictionary *errorInfo;
	if(![scrpt compileAndReturnError:&errorInfo] && errorInfo){
		[scriptText setSelectedRange:NSMakeRange(0, 3200)];
		[scriptText insertText:[scrpt source]];
		[scriptText insertText:[errorInfo description]];
		[scrpt release];
		return;
	}
	if(![scrpt executeAndReturnError:&errorInfo] && errorInfo){
		[scriptText setSelectedRange:NSMakeRange(0, 3200)];
		[scriptText insertText:[scrpt source]];
		[scriptText insertText:[errorInfo description]];
		[scrpt release];
		return;
	}
	[scrpt release];
	
	[self backPage];
}

@end
